/*      -*- OpenSAF  -*-
 *
 * (C) Copyright 2008 The OpenSAF Foundation
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. This file and program are licensed
 * under the GNU Lesser General Public License Version 2.1, February 1999.
 * The complete license can be accessed from the following location:
 * http://opensource.org/licenses/lgpl-license.php
 * See the Copying file included with the OpenSAF distribution for full
 * licensing terms.
 *
 * Author(s): Ericsson AB
 *
 */

#ifndef LOG_LOGD_LGS_H_
#define LOG_LOGD_LGS_H_

/* ========================================================================
 *   INCLUDE FILES
 * ========================================================================
 */

#include <cstdlib>
#include <stdint.h>

#include "base/ncs_mda_pvt.h"
#include "mbc/mbcsv_papi.h"
#include "base/ncs_edu_pub.h"
#include "base/ncs_util.h"
#include <saAis.h>
#include "base/saf_error.h"
#include <saImmOm.h>
#include <saAmf.h>

/* LGS files */
#include "log/common/lgsv_defs.h"
#include "lgs_cb.h"
#include "log/common/lgsv_msg.h"
#include "lgs_evt.h"
#include "lgs_mbcsv.h"
#include "log/logd/lgs_util.h"
#include "lgs_stream.h"
#include "lgs_config.h"
#include "lgs_imm_gcfg.h"

/* ========================================================================
 *   DEFINITIONS
 * ========================================================================
 */

/* Hard Coded in test!!! FIX */
#define ALARM_LOG_STREAM_ID 0
#define NOTIFICATION_LOG_STREAM_ID 1
#define SYSTEM_LOG_STREAM_ID 2
#define APPLICATION_LOG_STREAM_ID 3
#define LGS_FORMAT_EXP_MAX 256
#define DEFAULT_FIXED_LOG_RECORD_SIZE 150
#define DEFAULT_ALM_ACTION "ROTATE"
#define DEFAULT_ALM_ACTION_ROTATE 3
#define DEFAULT_MAX_LOG_FILE_SIZE 500000
#define DEFAULT_MAX_FILES_ROTATED 4
#define DEFAULT_APP_MAX_LOG_FILE_SIZE 800000
#define DEFAULT_APP_NUM_ROTATIONS 4

/* IPC priorities used when adding messages to the LOG mailbox */
#define LGS_IPC_PRIO_CTRL_MSGS NCS_IPC_PRIORITY_VERY_HIGH
#define LGS_IPC_PRIO_ALARM_STREAM NCS_IPC_PRIORITY_HIGH
#define LGS_IPC_PRIO_SYS_STREAM NCS_IPC_PRIORITY_NORMAL
#define LGS_IPC_PRIO_APP_STREAM NCS_IPC_PRIORITY_LOW

/* The name of log service config object */
#define LGS_IMM_LOG_CONFIGURATION "logConfig=1,safApp=safLogService"

/* The possible configurations for LGS_IMM_LOG_FILE_SYS_CONFIG */
#define LGS_LOG_SHARED_FILESYSTEM 1 /* Use shared filesystem. Default */
#define LGS_LOG_SPLIT_FILESYSTEM          \
  2 /* Store logs on local file system on \
       each node */

const SaVersionT kAmfVersion = {'B', 0x01, 0x01};
const SaVersionT kClmVersion = {'B', 0x04, 0x01};
const SaVersionT kImmVersion = {'A', 0x02, 0x0B};

/* ========================================================================
 *   DATA DECLARATIONS
 * ========================================================================
 */
extern lgs_cb_t *lgs_cb;
extern SYSF_MBX lgs_mbx;
extern uint32_t mbox_high[NCS_IPC_PRIORITY_MAX];
extern uint32_t mbox_msgs[NCS_IPC_PRIORITY_MAX];
extern bool mbox_full[NCS_IPC_PRIORITY_MAX];
extern uint32_t mbox_low[NCS_IPC_PRIORITY_MAX];
extern pthread_mutex_t lgs_mbox_init_mutex;

extern uint32_t initialize_for_assignment(lgs_cb_t *cb, SaAmfHAStateT ha_state);

extern uint32_t lgs_configure_mailbox();

extern uint32_t lgs_mds_init(lgs_cb_t *cb, SaAmfHAStateT ha_state);
extern uint32_t lgs_mds_finalize(lgs_cb_t *cb);
extern uint32_t lgs_mds_change_role(lgs_cb_t *cb);
extern uint32_t lgs_mds_msg_send(lgs_cb_t *cb, lgsv_msg_t *msg, MDS_DEST *dest,
                                 MDS_SYNC_SND_CTXT *mds_ctxt,
                                 MDS_SEND_PRIORITY_TYPE prio);

#endif  // LOG_LOGD_LGS_H_
